#ifndef GRAPH__H_
#define GRAPH__H_

/**
  * @file GRAPH.h
  *
  * Library to draw graphs of mathematical functions in Bitmaps.
  *
  * @author Marcell Dietl, Sven Eric Panitz
  * @date 25.01.2010
  *
  **/

/**
  * Fill the background of a Bitmap with a given color.
  *
  * The function depends on the fillRect() function to
  * draw a rectangle, starting from the bottom left of the
  * Bitmap to the upper right.
  *
  * @param this Pointer to the Bitmap in memory.
  * @param c Color to be used for the background.
  *
  **/
void background(Bmp* this, Color c);

/**
  * Draw a rectangle with a given color.
  *
  * The function draws a rectangle - starting from point x/y
  * with the width w and the height h.
  *
  * @param this Pointer to the Bitmap in memory.
  * @param c Color to be used for the rectangle's background.
  * @param x Starting point on the X-axis.
  * @param y Starting point on the Y-axis.
  * @param w Width of the rectangle.
  * @param h Height of the rectangle.
  *
  **/
void fillRect(Bmp* this, Color c, int x, int y, int w, int h);

/* Copied from skript.pdf *************************************************/

/**
  * Swap the values of two integers in memory.
  *
  * The function is only used by the drawLine() function to swap
  * the value saved in x with the one in y by simply manipulating the
  * memory pointers.
  *
  * @param x The value to be saved in y afterward.
  * @param y The value to be saved in x afterward.
  *
  **/
void swapValues(int* x,int* y);

/**
  * Draw a thin line from one point to another.
  *
  * The function draws a line with a given color from the point fromX/fromY
  * to the point toX/toY  - allowing any possible coordinates.
  *
  * @param this Pointer to the Bitmap in memory.
  * @param c Color to be used for the drawn line.
  * @param fromX Starting point on the X-axis.
  * @param fromY Starting point on the Y-axis.
  * @param toX End point on the X-axis.
  * @param toY End point on the Y-axis.
  *
  **/
void drawLine(Bmp* this, Color c, int fromX, int fromY, int toX, int toY);

/**************************************************************************/

/**
  * Generate a Bitmap showing a function's graph.
  *
  * The function draws the graph of a given mathematical function into
  * a coordinate system dynamically. Thus allowing any possible input.
  *
  * @param f Mathemtical function to be drawn.
  * @param minX Smallest value on the X-axis.
  * @param maxX Greatest value on the X-axis.
  *
  * @result Pointer to the generated Bitmap in memory.
  *
  **/
Bmp* drawGraph(double (f)(double), double minX, double maxX);

/**
  * Calculate the derivative of a function.
  *
  * The function calculates the value of a point x on
  * the derivative of a given mathematical function.
  *
  * @param f Mathematical function to be derivated.
  * @param x Value to be calculated on the derivative of f.
  *
  * @result Value of point x on the derivative of f.
  *
  **/
double ab(double (f)(double), double x);

/**
  * Calculate the n-th derivative of a function.
  *
  * The function recursively calculates the value of a point x on
  * the n-th derivative of a given mathematical function.
  *
  * @param f Mathematical function to be derivated.
  * @param x Value to be calculated on the n-th derivative of f.
  * @param n Number of derivatives.
  *
  * @result Value of point x on the n-th derivative of f.
  *
  **/
double abN(double (f)(double), double x, unsigned int n);

/**
  * Generate a Bitmap showing a function's graph with derivatives.
  *
  * The function draws the graph of a given mathematical function - including
  * numberOfDiffs of the function's derivatives - into a coordinate system
  * dynamically. Thus allowing any possible input.
  *
  * @param f Mathematical function to be drawn.
  * @param minX Smallest value on the X-axis.
  * @param maxX Greatest value on the X-axis.
  * @param numberOfDiffs Number of derivatives to be drawn.
  *
  * @result Pointer to the generated Bitmap in memory.
  *
  **/
Bmp* drawGraphWithDiffs(double (f)(double), double minX, double maxX, unsigned int numberOfDiffs);

#endif
